@@ -12,6 +12,8 @@
 #include "e-data-book.h"
 #include "e-book-backend.h"
 
+G_DEFINE_TYPE (EBookBackend, e_book_backend, G_TYPE_OBJECT)
+
 struct _EBookBackendPrivate {
 	GMutex *open_mutex;
 
@@ -41,7 +43,7 @@ static GObjectClass *parent_class;
  *
  * Does nothing.
  *
- * Return value: %TRUE.
+ * Returns: %TRUE.
  **/
 gboolean
 e_book_backend_construct (EBookBackend *backend)
@@ -57,7 +59,7 @@ e_book_backend_construct (EBookBackend *backend)
  *
  * Loads @source into @backend.
  *
- * Return value: A #GNOME_Evolution_Addressbook_CallStatus indicating the outcome.
+ * Returns: A #GNOME_Evolution_Addressbook_CallStatus indicating the outcome.
  **/
 GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_load_source (EBookBackend           *backend,
@@ -88,7 +90,7 @@ e_book_backend_load_source (EBookBackend           *backend,
  *
  * Queries the source that an addressbook backend is serving.
  *
- * Return value: ESource for the backend.
+ * Returns: ESource for the backend.
  **/
 ESource *
 e_book_backend_get_source (EBookBackend *backend)
@@ -120,20 +122,20 @@ e_book_backend_open (EBookBackend *backend,
 	g_mutex_lock (backend->priv->open_mutex);
 
 	if (backend->priv->loaded) {
-		e_data_book_respond_open (
-			book, opid, GNOME_Evolution_Addressbook_Success);
-
 		e_data_book_report_writable (book, backend->priv->writable);
 		e_data_book_report_connection_status (book, backend->priv->online);
+
+		e_data_book_respond_open (
+			book, opid, GNOME_Evolution_Addressbook_Success);
 	} else {
 		GNOME_Evolution_Addressbook_CallStatus status =
 			e_book_backend_load_source (backend, e_data_book_get_source (book), only_if_exists);
 
-		e_data_book_respond_open (book, opid, status);
-
 		if (status == GNOME_Evolution_Addressbook_Success || status == GNOME_Evolution_Addressbook_InvalidServerVersion)
 			e_data_book_report_writable (book, backend->priv->writable);
 			e_data_book_report_connection_status (book, backend->priv->online);
+
+		e_data_book_respond_open (book, opid, status);
 	}
 
 	g_mutex_unlock (backend->priv->open_mutex);
@@ -455,7 +457,7 @@ e_book_backend_get_supported_auth_methods (EBookBackend *backend,
  *
  * Cancel @book's running operation on @backend.
  *
- * Return value: A GNOME_Evolution_Addressbook_CallStatus indicating the outcome.
+ * Returns: A GNOME_Evolution_Addressbook_CallStatus indicating the outcome.
  **/
 GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_cancel_operation (EBookBackend *backend,
@@ -481,7 +483,7 @@ last_client_gone (EBookBackend *backend)
  *
  * Gets the list of #EDataBookView views running on this backend.
  *
- * Return value: An #EList of #EDataBookView objects.
+ * Returns: An #EList of #EDataBookView objects.
  **/
 EList*
 e_book_backend_get_book_views (EBookBackend *backend)
@@ -538,7 +540,7 @@ e_book_backend_remove_book_view (EBookBackend *backend,
  *
  * Adds a client to an addressbook backend.
  *
- * Return value: TRUE on success, FALSE on failure to add the client.
+ * Returns: TRUE on success, FALSE on failure to add the client.
  */
 gboolean
 e_book_backend_add_client (EBookBackend      *backend,
@@ -594,7 +596,7 @@ e_book_backend_remove_client (EBookBackend *backend,
  *
  * Checks if @backend has clients running in other system processes.
  *
- * Return value: %TRUE if there are clients in other processes, %FALSE otherwise.
+ * Returns: %TRUE if there are clients in other processes, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_has_out_of_proc_clients (EBookBackend *backend)
@@ -608,7 +610,7 @@ e_book_backend_has_out_of_proc_clients (EBookBackend *backend)
  *
  * Gets the capabilities offered by this @backend.
  *
- * Return value: A string listing the capabilities.
+ * Returns: A string listing the capabilities.
  **/
 gchar *
 e_book_backend_get_static_capabilities (EBookBackend *backend)
@@ -627,7 +629,7 @@ e_book_backend_get_static_capabilities (EBookBackend *backend)
  * Checks if @backend's storage has been opened and the backend
  * itself is ready for accessing.
  *
- * Return value: %TRUE if loaded, %FALSE otherwise.
+ * Returns: %TRUE if loaded, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_is_loaded (EBookBackend *backend)
@@ -659,7 +661,7 @@ e_book_backend_set_is_loaded (EBookBackend *backend, gboolean is_loaded)
  *
  * Checks if we can write to @backend.
  *
- * Return value: %TRUE if writeable, %FALSE if not.
+ * Returns: %TRUE if writeable, %FALSE if not.
  **/
 gboolean
 e_book_backend_is_writable (EBookBackend *backend)
@@ -691,7 +693,7 @@ e_book_backend_set_is_writable (EBookBackend *backend, gboolean is_writable)
  *
  * Checks if @backend has been removed from its physical storage.
  *
- * Return value: %TRUE if @backend has been removed, %FALSE otherwise.
+ * Returns: %TRUE if @backend has been removed, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_is_removed (EBookBackend *backend)
@@ -744,6 +746,8 @@ e_book_backend_set_mode (EBookBackend *backend,
  * Write all pending data to disk.  This is only required under special
  * circumstances (for example before a live backup) and should not be used in
  * normal use.
+ *
+ * Since: 1.12
  */
 void
 e_book_backend_sync (EBookBackend *backend)
@@ -761,7 +765,7 @@ e_book_backend_sync (EBookBackend *backend)
  * Creates a new change item indicating @vcard was added.
  * Meant to be used by backend implementations.
  *
- * Return value: A new #GNOME_Evolution_Addressbook_BookChangeItem.
+ * Returns: A new #GNOME_Evolution_Addressbook_BookChangeItem.
  **/
 EDataBookChange *
 e_book_backend_change_add_new     (const gchar *vcard)
@@ -781,7 +785,7 @@ e_book_backend_change_add_new     (const gchar *vcard)
  * Creates a new change item indicating @vcard was modified.
  * Meant to be used by backend implementations.
  *
- * Return value: A new #GNOME_Evolution_Addressbook_BookChangeItem.
+ * Returns: A new #GNOME_Evolution_Addressbook_BookChangeItem.
  **/
 EDataBookChange *
 e_book_backend_change_modify_new  (const gchar *vcard)
@@ -801,7 +805,7 @@ e_book_backend_change_modify_new  (const gchar *vcard)
  * Creates a new change item indicating @vcard was deleted.
  * Meant to be used by backend implementations.
  *
- * Return value: A new #GNOME_Evolution_Addressbook_BookChangeItem.
+ * Returns: A new #GNOME_Evolution_Addressbook_BookChangeItem.
  **/
 EDataBookChange *
 e_book_backend_change_delete_new  (const gchar *vcard)
@@ -1039,7 +1043,7 @@ e_book_backend_class_init (EBookBackendClass *klass)
 	object_class->dispose = e_book_backend_dispose;
 
 	e_book_backend_signals[LAST_CLIENT_GONE] =
-		g_signal_new ("last_client_gone",
+		g_signal_new ("last-client-gone",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EBookBackendClass, last_client_gone),
@@ -1047,30 +1051,3 @@ e_book_backend_class_init (EBookBackendClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 }
-
-/**
- * e_book_backend_get_type:
- */
-GType
-e_book_backend_get_type (void)
-{
-	static GType type = 0;
-
-	if (! type) {
-		GTypeInfo info = {
-			sizeof (EBookBackendClass),
-			NULL, /* base_class_init */
-			NULL, /* base_class_finalize */
-			(GClassInitFunc)  e_book_backend_class_init,
-			NULL, /* class_finalize */
-			NULL, /* class_data */
-			sizeof (EBookBackend),
-			0,    /* n_preallocs */
-			(GInstanceInitFunc) e_book_backend_init
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT, "EBookBackend", &info, 0);
-	}
-
-	return type;
-}
