@@ -25,9 +25,10 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <libebook/e-contact.h>
-#include <libedata-book/Evolution-DataServer-Addressbook.h>
 #include <libedata-book/e-data-book-types.h>
 #include <libedata-book/e-data-book.h>
+#include <libedataserver/e-list.h>
+#include <libedataserver/e-source.h>
 
 G_BEGIN_DECLS
 
@@ -66,7 +67,7 @@ struct _EBookBackendClass {
 	void (*get_supported_fields) (EBookBackend *backend, EDataBook *book, guint32 opid);
 	void (*get_supported_auth_methods) (EBookBackend *backend, EDataBook *book, guint32 opid);
 	GNOME_Evolution_Addressbook_CallStatus (*cancel_operation) (EBookBackend *backend, EDataBook *book);
-	void (*set_mode) (EBookBackend *backend, GNOME_Evolution_Addressbook_BookMode  mode);
+	void (*set_mode) (EBookBackend *backend, GNOME_Evolution_Addressbook_BookMode mode);
 
 	/* Notification signals */
 	void (* last_client_gone) (EBookBackend *backend);
@@ -179,7 +180,6 @@ void        e_book_backend_sync                       (EBookBackend *backend);
 
 GType       e_book_backend_get_type                 (void);
 
-
 /* protected functions for subclasses */
 void        e_book_backend_set_is_loaded            (EBookBackend             *backend,
 						     gboolean                  is_loaded);
@@ -195,5 +195,5 @@ GNOME_Evolution_Addressbook_BookChangeItem* e_book_backend_change_delete_new  (c
 
 G_END_DECLS
 
-#endif /* ! __E_BOOK_BACKEND_H__ */
+#endif /* __E_BOOK_BACKEND_H__ */
 
