@@ -29,19 +29,27 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <glib.h>
 #include <glib/gi18n-lib.h>
 
 #include "camel-charset-map.h"
 #include "camel-iconv.h"
 #include "camel-mime-utils.h"
 #include "camel-net-utils.h"
+#ifdef G_OS_WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#include "camel-net-utils-win32.h"
+#endif
 #include "camel-sasl-digest-md5.h"
 
 #define d(x)
 
 #define PARANOID(x) x
 
+#define CAMEL_SASL_DIGEST_MD5_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), CAMEL_TYPE_SASL_DIGEST_MD5, CamelSaslDigestMd5Private))
+
 /* Implements rfc2831 */
 
 CamelServiceAuthType camel_sasl_digest_md5_authtype = {
@@ -54,13 +62,6 @@ CamelServiceAuthType camel_sasl_digest_md5_authtype = {
 	TRUE
 };
 
-static CamelSaslClass *parent_class = NULL;
-
-/* Returns the class for a CamelSaslDigestMd5 */
-#define CSCM_CLASS(so) CAMEL_SASL_DIGEST_MD5_CLASS (CAMEL_OBJECT_GET_CLASS (so))
-
-static GByteArray *digest_md5_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex);
-
 enum {
 	STATE_AUTH,
 	STATE_FINAL
@@ -168,90 +169,7 @@ struct _CamelSaslDigestMd5Private {
 	gint state;
 };
 
-static void
-camel_sasl_digest_md5_class_init (CamelSaslDigestMd5Class *camel_sasl_digest_md5_class)
-{
-	CamelSaslClass *camel_sasl_class = CAMEL_SASL_CLASS (camel_sasl_digest_md5_class);
-
-	parent_class = CAMEL_SASL_CLASS (camel_type_get_global_classfuncs (camel_sasl_get_type ()));
-
-	/* virtual method overload */
-	camel_sasl_class->challenge = digest_md5_challenge;
-}
-
-static void
-camel_sasl_digest_md5_init (gpointer object, gpointer klass)
-{
-	CamelSaslDigestMd5 *sasl_digest = CAMEL_SASL_DIGEST_MD5 (object);
-
-	sasl_digest->priv = g_new0 (struct _CamelSaslDigestMd5Private, 1);
-}
-
-static void
-camel_sasl_digest_md5_finalize (CamelObject *object)
-{
-	CamelSaslDigestMd5 *sasl = CAMEL_SASL_DIGEST_MD5 (object);
-	struct _DigestChallenge *c = sasl->priv->challenge;
-	struct _DigestResponse *r = sasl->priv->response;
-	GList *p;
-	gint i;
-
-	if (c != NULL) {
-		for (i = 0; i < c->realms->len; i++)
-			g_free (c->realms->pdata[i]);
-		g_ptr_array_free (c->realms, TRUE);
-
-		g_free (c->nonce);
-		g_free (c->charset);
-		g_free (c->algorithm);
-		for (p = c->params; p; p = p->next) {
-			struct _param *param = p->data;
-
-			g_free (param->name);
-			g_free (param->value);
-			g_free (param);
-		}
-		g_list_free (c->params);
-		g_free (c);
-	}
-
-	if (r != NULL) {
-		g_free (r->username);
-		g_free (r->realm);
-		g_free (r->nonce);
-		g_free (r->cnonce);
-		if (r->uri) {
-			g_free (r->uri->type);
-			g_free (r->uri->host);
-		g_free (r->uri->name);
-		}
-		g_free (r->charset);
-		g_free (r->authzid);
-		g_free (r->param);
-		g_free (r);
-	}
-
-	g_free (sasl->priv);
-}
-
-CamelType
-camel_sasl_digest_md5_get_type (void)
-{
-	static CamelType type = CAMEL_INVALID_TYPE;
-
-	if (type == CAMEL_INVALID_TYPE) {
-		type = camel_type_register (camel_sasl_get_type (),
-					    "CamelSaslDigestMd5",
-					    sizeof (CamelSaslDigestMd5),
-					    sizeof (CamelSaslDigestMd5Class),
-					    (CamelObjectClassInitFunc) camel_sasl_digest_md5_class_init,
-					    NULL,
-					    (CamelObjectInitFunc) camel_sasl_digest_md5_init,
-					    (CamelObjectFinalizeFunc) camel_sasl_digest_md5_finalize);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (CamelSaslDigestMd5, camel_sasl_digest_md5, CAMEL_TYPE_SASL)
 
 static void
 decode_lwsp (const gchar **in)
@@ -818,11 +736,62 @@ digest_response (struct _DigestResponse *resp)
 	return buffer;
 }
 
+static void
+sasl_digest_md5_finalize (GObject *object)
+{
+	CamelSaslDigestMd5 *sasl = CAMEL_SASL_DIGEST_MD5 (object);
+	struct _DigestChallenge *c = sasl->priv->challenge;
+	struct _DigestResponse *r = sasl->priv->response;
+	GList *p;
+	gint i;
+
+	if (c != NULL) {
+		for (i = 0; i < c->realms->len; i++)
+			g_free (c->realms->pdata[i]);
+		g_ptr_array_free (c->realms, TRUE);
+
+		g_free (c->nonce);
+		g_free (c->charset);
+		g_free (c->algorithm);
+		for (p = c->params; p; p = p->next) {
+			struct _param *param = p->data;
+
+			g_free (param->name);
+			g_free (param->value);
+			g_free (param);
+		}
+		g_list_free (c->params);
+		g_free (c);
+	}
+
+	if (r != NULL) {
+		g_free (r->username);
+		g_free (r->realm);
+		g_free (r->nonce);
+		g_free (r->cnonce);
+		if (r->uri) {
+			g_free (r->uri->type);
+			g_free (r->uri->host);
+		g_free (r->uri->name);
+		}
+		g_free (r->charset);
+		g_free (r->authzid);
+		g_free (r->param);
+		g_free (r);
+	}
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_sasl_digest_md5_parent_class)->finalize (object);
+}
+
 static GByteArray *
-digest_md5_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
+sasl_digest_md5_challenge (CamelSasl *sasl,
+                           GByteArray *token,
+                           CamelException *ex)
 {
 	CamelSaslDigestMd5 *sasl_digest = CAMEL_SASL_DIGEST_MD5 (sasl);
 	struct _CamelSaslDigestMd5Private *priv = sasl_digest->priv;
+	CamelService *service;
 	struct _param *rspauth;
 	GByteArray *ret = NULL;
 	gboolean abort = FALSE;
@@ -830,18 +799,22 @@ digest_md5_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 	guchar out[33];
 	gchar *tokens;
 	struct addrinfo *ai, hints;
+	const gchar *service_name;
 
 	/* Need to wait for the server */
 	if (!token)
 		return NULL;
 
-	g_return_val_if_fail (sasl->service->url->passwd != NULL, NULL);
+	service = camel_sasl_get_service (sasl);
+	service_name = camel_sasl_get_service_name (sasl);
+	g_return_val_if_fail (service->url->passwd != NULL, NULL);
 
 	switch (priv->state) {
 	case STATE_AUTH:
 		if (token->len > 2048) {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					      _("Server challenge too long (>2048 octets)"));
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
+				_("Server challenge too long (>2048 octets)"));
 			return NULL;
 		}
 
@@ -849,29 +822,31 @@ digest_md5_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 		priv->challenge = parse_server_challenge (tokens, &abort);
 		g_free (tokens);
 		if (!priv->challenge || abort) {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					      _("Server challenge invalid\n"));
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
+				_("Server challenge invalid\n"));
 			return NULL;
 		}
 
 		if (priv->challenge->qop == QOP_INVALID) {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					      _("Server challenge contained invalid "
-						"\"Quality of Protection\" token"));
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
+				_("Server challenge contained invalid "
+				  "\"Quality of Protection\" token"));
 			return NULL;
 		}
 
 		memset(&hints, 0, sizeof(hints));
 		hints.ai_flags = AI_CANONNAME;
-		ai = camel_getaddrinfo(sasl->service->url->host?sasl->service->url->host:"localhost", NULL, &hints, NULL);
+		ai = camel_getaddrinfo(service->url->host?service->url->host:"localhost", NULL, &hints, NULL);
 		if (ai && ai->ai_canonname)
 			ptr = ai->ai_canonname;
 		else
 			ptr = "localhost.localdomain";
 
-		priv->response = generate_response (priv->challenge, ptr, sasl->service_name,
-						    sasl->service->url->user,
-						    sasl->service->url->passwd);
+		priv->response = generate_response (priv->challenge, ptr, service_name,
+						    service->url->user,
+						    service->url->passwd);
 		if (ai)
 			camel_freeaddrinfo(ai);
 		ret = digest_response (priv->response);
@@ -885,8 +860,10 @@ digest_md5_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 
 		if (!tokens || !*tokens) {
 			g_free (tokens);
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					      _("Server response did not contain authorization data"));
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
+				_("Server response did not contain "
+				  "authorization data"));
 			return NULL;
 		}
 
@@ -903,19 +880,22 @@ digest_md5_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 		if (!rspauth->value) {
 			g_free (rspauth->name);
 			g_free (rspauth);
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					      _("Server response contained incomplete authorization data"));
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
+				_("Server response contained incomplete "
+				  "authorization data"));
 			return NULL;
 		}
 
-		compute_response (priv->response, sasl->service->url->passwd, FALSE, out);
+		compute_response (priv->response, service->url->passwd, FALSE, out);
 		if (memcmp (out, rspauth->value, 32) != 0) {
 			g_free (rspauth->name);
 			g_free (rspauth->value);
 			g_free (rspauth);
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-					      _("Server response does not match"));
-			sasl->authenticated = TRUE;
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
+				_("Server response does not match"));
+			camel_sasl_set_authenticated (sasl, TRUE);
 
 			return NULL;
 		}
@@ -926,7 +906,7 @@ digest_md5_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 
 		ret = g_byte_array_new ();
 
-		sasl->authenticated = TRUE;
+		camel_sasl_set_authenticated (sasl, TRUE);
 	default:
 		break;
 	}
@@ -935,3 +915,24 @@ digest_md5_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 
 	return ret;
 }
+
+static void
+camel_sasl_digest_md5_class_init (CamelSaslDigestMd5Class *class)
+{
+	GObjectClass *object_class;
+	CamelSaslClass *sasl_class;
+
+	g_type_class_add_private (class, sizeof (CamelSaslDigestMd5Private));
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->finalize = sasl_digest_md5_finalize;
+
+	sasl_class = CAMEL_SASL_CLASS (class);
+	sasl_class->challenge = sasl_digest_md5_challenge;
+}
+
+static void
+camel_sasl_digest_md5_init (CamelSaslDigestMd5 *sasl)
+{
+	sasl->priv = CAMEL_SASL_DIGEST_MD5_GET_PRIVATE (sasl);
+}
