@@ -33,12 +33,6 @@
 #include <unistd.h>
 #include <sys/stat.h>
 
-#include "camel-db.h"
-#include "camel-data-cache.h"
-#include "camel-file-utils.h"
-#include "camel-folder.h"
-#include "camel-string-utils.h"
-
 #include "camel-groupwise-folder.h"
 #include "camel-groupwise-summary.h"
 
@@ -53,11 +47,9 @@
 static gint gw_summary_header_load (CamelFolderSummary *, FILE *);
 static gint gw_summary_header_save (CamelFolderSummary *, FILE *);
 
-static CamelMessageInfo *gw_message_info_load (CamelFolderSummary *s, FILE *in);
+static CamelMessageInfo *gw_message_info_migrate (CamelFolderSummary *s, FILE *in);
 
-static gint gw_message_info_save (CamelFolderSummary *s, FILE *out, CamelMessageInfo *info);
-static CamelMessageContentInfo * gw_content_info_load (CamelFolderSummary *s, FILE *in);
-static gint gw_content_info_save (CamelFolderSummary *s, FILE *out, CamelMessageContentInfo *info);
+static CamelMessageContentInfo * gw_content_info_migrate (CamelFolderSummary *s, FILE *in);
 static gboolean gw_info_set_flags(CamelMessageInfo *info, guint32 flags, guint32 set);
 
 static gint summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *mir);
@@ -67,34 +59,9 @@ static CamelMessageInfo * message_info_from_db (CamelFolderSummary *s, CamelMIRe
 static gint content_info_to_db (CamelFolderSummary *s, CamelMessageContentInfo *info, CamelMIRecord *mir);
 static CamelMessageContentInfo * content_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir);
 
-static void camel_groupwise_summary_class_init (CamelGroupwiseSummaryClass *klass);
-static void camel_groupwise_summary_init       (CamelGroupwiseSummary *obj);
-
-
 /*End of Prototypes*/
 
-
-static CamelFolderSummaryClass *camel_groupwise_summary_parent;
-
-
-CamelType
-camel_groupwise_summary_get_type (void)
-{
-	static CamelType type = CAMEL_INVALID_TYPE;
-
-	if (type == CAMEL_INVALID_TYPE) {
-		type = camel_type_register(
-				camel_folder_summary_get_type(), "CamelGroupwiseSummary",
-				sizeof (CamelGroupwiseSummary),
-				sizeof (CamelGroupwiseSummaryClass),
-				(CamelObjectClassInitFunc) camel_groupwise_summary_class_init,
-				NULL,
-				(CamelObjectInitFunc) camel_groupwise_summary_init,
-				NULL);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (CamelGroupwiseSummary, camel_groupwise_summary, CAMEL_TYPE_FOLDER_SUMMARY)
 
 static CamelMessageInfo *
 gw_message_info_clone(CamelFolderSummary *s, const CamelMessageInfo *mi)
@@ -102,7 +69,7 @@ gw_message_info_clone(CamelFolderSummary *s, const CamelMessageInfo *mi)
 	CamelGroupwiseMessageInfo *to;
 	const CamelGroupwiseMessageInfo *from = (const CamelGroupwiseMessageInfo *)mi;
 
-	to = (CamelGroupwiseMessageInfo *)camel_groupwise_summary_parent->message_info_clone(s, mi);
+	to = (CamelGroupwiseMessageInfo *)CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->message_info_clone(s, mi);
 	to->server_flags = from->server_flags;
 
 	/* FIXME: parent clone should do this */
@@ -112,45 +79,36 @@ gw_message_info_clone(CamelFolderSummary *s, const CamelMessageInfo *mi)
 }
 
 static void
-camel_groupwise_summary_class_init (CamelGroupwiseSummaryClass *klass)
+camel_groupwise_summary_class_init (CamelGroupwiseSummaryClass *class)
 {
-	CamelFolderSummaryClass *cfs_class = (CamelFolderSummaryClass *) klass;
-
-	camel_groupwise_summary_parent = CAMEL_FOLDER_SUMMARY_CLASS (camel_type_get_global_classfuncs (camel_folder_summary_get_type()));
-
-	cfs_class->message_info_clone = gw_message_info_clone;
-	cfs_class->summary_header_load = gw_summary_header_load;
-	cfs_class->summary_header_save = gw_summary_header_save;
-	cfs_class->message_info_load = gw_message_info_load;
-	cfs_class->message_info_save = gw_message_info_save;
-	cfs_class->content_info_load = gw_content_info_load;
-	cfs_class->content_info_save = gw_content_info_save;
-	cfs_class->info_set_flags = gw_info_set_flags;
-
-	cfs_class->summary_header_to_db = summary_header_to_db;
-	cfs_class->summary_header_from_db = summary_header_from_db;
-	cfs_class->message_info_to_db = message_info_to_db;
-	cfs_class->message_info_from_db = message_info_from_db;
-	cfs_class->content_info_to_db = content_info_to_db;
-	cfs_class->content_info_from_db = content_info_from_db;
+	CamelFolderSummaryClass *folder_summary_class;
 
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (class);
+	folder_summary_class->message_info_size = sizeof (CamelGroupwiseMessageInfo);
+	folder_summary_class->content_info_size = sizeof (CamelGroupwiseMessageContentInfo);
+	folder_summary_class->message_info_clone = gw_message_info_clone;
+	folder_summary_class->summary_header_load = gw_summary_header_load;
+	folder_summary_class->summary_header_save = gw_summary_header_save;
+	folder_summary_class->message_info_migrate = gw_message_info_migrate;
+	folder_summary_class->content_info_migrate = gw_content_info_migrate;
+	folder_summary_class->info_set_flags = gw_info_set_flags;
+	folder_summary_class->summary_header_to_db = summary_header_to_db;
+	folder_summary_class->summary_header_from_db = summary_header_from_db;
+	folder_summary_class->message_info_to_db = message_info_to_db;
+	folder_summary_class->message_info_from_db = message_info_from_db;
+	folder_summary_class->content_info_to_db = content_info_to_db;
+	folder_summary_class->content_info_from_db = content_info_from_db;
 }
 
-
 static void
-camel_groupwise_summary_init (CamelGroupwiseSummary *obj)
+camel_groupwise_summary_init (CamelGroupwiseSummary *gw_summary)
 {
-	CamelFolderSummary *s = (CamelFolderSummary *)obj;
-
-	/* subclasses need to set the right instance data sizes */
-	s->message_info_size = sizeof(CamelGroupwiseMessageInfo);
-	s->content_info_size = sizeof(CamelGroupwiseMessageContentInfo);
+	CamelFolderSummary *summary = CAMEL_FOLDER_SUMMARY (gw_summary);
 
 	/* Meta-summary - Overriding UID len */
-	s->meta_summary->uid_len = 2048;
+	summary->meta_summary->uid_len = 2048;
 }
 
-
 /**
  * camel_groupwise_summary_new:
  * @filename: the file to store the summary in.
@@ -158,15 +116,15 @@ camel_groupwise_summary_init (CamelGroupwiseSummary *obj)
  * This will create a new CamelGroupwiseSummary object and read in the
  * summary data from disk, if it exists.
  *
- * Return value: A new CamelGroupwiseSummary object.
+ * Returns: A new CamelGroupwiseSummary object.
  **/
 CamelFolderSummary *
 camel_groupwise_summary_new (struct _CamelFolder *folder, const gchar *filename)
 {
+	CamelFolderSummary *summary;
 	CamelException ex;
-	CamelFolderSummary *summary = CAMEL_FOLDER_SUMMARY (
-			camel_object_new (camel_groupwise_summary_get_type ()));
 
+	summary = g_object_new (CAMEL_TYPE_GROUPWISE_SUMMARY, NULL);
 	summary->folder = folder;
 	camel_folder_summary_set_build_content (summary, TRUE);
 	camel_folder_summary_set_filename (summary, filename);
@@ -185,7 +143,7 @@ summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *mir)
 	CamelGroupwiseSummary *gms = CAMEL_GROUPWISE_SUMMARY (s);
 	gchar *part;
 
-	if (camel_groupwise_summary_parent->summary_header_from_db (s, mir) == -1)
+	if (CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->summary_header_from_db (s, mir) == -1)
 		return -1;
 
 	part = mir->bdata;
@@ -208,7 +166,7 @@ gw_summary_header_load (CamelFolderSummary *s, FILE *in)
 {
 	CamelGroupwiseSummary *gms = CAMEL_GROUPWISE_SUMMARY (s);
 
-	if (camel_groupwise_summary_parent->summary_header_load (s, in) == -1)
+	if (CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->summary_header_load (s, in) == -1)
 		return -1;
 
 	if (camel_file_util_decode_fixed_int32(in, &gms->version) == -1
@@ -220,17 +178,13 @@ gw_summary_header_load (CamelFolderSummary *s, FILE *in)
 	return 0;
 }
 
-
-
-
-
 static CamelFIRecord *
 summary_header_to_db (CamelFolderSummary *s, CamelException *ex)
 {
 	CamelGroupwiseSummary *ims = CAMEL_GROUPWISE_SUMMARY(s);
 	struct _CamelFIRecord *fir;
 
-	fir = camel_groupwise_summary_parent->summary_header_to_db (s, ex);
+	fir = CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->summary_header_to_db (s, ex);
 	if (!fir)
 		return NULL;
 
@@ -245,7 +199,7 @@ gw_summary_header_save (CamelFolderSummary *s, FILE *out)
 {
 	CamelGroupwiseSummary *gms = CAMEL_GROUPWISE_SUMMARY(s);
 
-	if (camel_groupwise_summary_parent->summary_header_save (s, out) == -1)
+	if (CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->summary_header_save (s, out) == -1)
 		return -1;
 
 	camel_file_util_encode_fixed_int32(out, CAMEL_GW_SUMMARY_VERSION);
@@ -259,7 +213,7 @@ message_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir)
 	CamelMessageInfo *info;
 	CamelGroupwiseMessageInfo *iinfo;
 
-	info = camel_groupwise_summary_parent->message_info_from_db (s, mir);
+	info = CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->message_info_from_db (s, mir);
 	if (info) {
 		gchar *part = mir->bdata;
 		iinfo = (CamelGroupwiseMessageInfo *)info;
@@ -269,13 +223,12 @@ message_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir)
 	return info;}
 
 static CamelMessageInfo *
-gw_message_info_load (CamelFolderSummary *s, FILE *in)
+gw_message_info_migrate (CamelFolderSummary *s, FILE *in)
 {
 	CamelMessageInfo *info;
 	CamelGroupwiseMessageInfo *gw_info;
 
-
-	info = camel_groupwise_summary_parent->message_info_load(s,in);
+	info = CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->message_info_migrate (s,in);
 	if (info) {
 		gw_info = (CamelGroupwiseMessageInfo*) info;
 		if (camel_file_util_decode_uint32 (in, &gw_info->server_flags) == -1)
@@ -294,24 +247,13 @@ message_info_to_db (CamelFolderSummary *s, CamelMessageInfo *info)
 	CamelGroupwiseMessageInfo *iinfo = (CamelGroupwiseMessageInfo *)info;
 	struct _CamelMIRecord *mir;
 
-	mir = camel_groupwise_summary_parent->message_info_to_db (s, info);
+	mir = CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->message_info_to_db (s, info);
 	if (mir)
 		mir->bdata = g_strdup_printf ("%u", iinfo->server_flags);
 
 	return mir;
 }
 
-static gint
-gw_message_info_save (CamelFolderSummary *s, FILE *out, CamelMessageInfo *info)
-{
-	CamelGroupwiseMessageInfo *gw_info = (CamelGroupwiseMessageInfo *)info;
-
-	if (camel_groupwise_summary_parent->message_info_save (s, out, info) == -1)
-		return -1;
-
-	return camel_file_util_encode_uint32 (out, gw_info->server_flags);
-}
-
 static CamelMessageContentInfo *
 content_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir)
 {
@@ -327,17 +269,16 @@ content_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir)
 	}
 	mir->cinfo = part;
 	if (type)
-		return camel_groupwise_summary_parent->content_info_from_db (s, mir);
+		return CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->content_info_from_db (s, mir);
 	else
 		return camel_folder_summary_content_info_new (s);
 }
 
-
 static CamelMessageContentInfo *
-gw_content_info_load (CamelFolderSummary *s, FILE *in)
+gw_content_info_migrate (CamelFolderSummary *s, FILE *in)
 {
 	if (fgetc (in))
-		return camel_groupwise_summary_parent->content_info_load (s, in);
+		return CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->content_info_migrate (s, in);
 	else
 		return camel_folder_summary_content_info_new (s);
 }
@@ -348,24 +289,13 @@ content_info_to_db (CamelFolderSummary *s, CamelMessageContentInfo *info, CamelM
 
 	if (info->type) {
 		mir->cinfo = g_strdup ("1");
-		return camel_groupwise_summary_parent->content_info_to_db (s, info, mir);
+		return CAMEL_FOLDER_SUMMARY_CLASS (camel_groupwise_summary_parent_class)->content_info_to_db (s, info, mir);
 	} else {
 		mir->cinfo = g_strdup ("0");
 		return 0;
 	}
 }
 
-static gint
-gw_content_info_save (CamelFolderSummary *s, FILE *out,
-		CamelMessageContentInfo *info)
-{
-	if (info->type) {
-		fputc (1, out);
-		return camel_groupwise_summary_parent->content_info_save (s, out, info);
-	} else
-		return fputc (0, out);
-}
-
 static gboolean
 gw_info_set_flags (CamelMessageInfo *info, guint32 flags, guint32 set)
 {
@@ -418,12 +348,11 @@ gw_info_set_flags (CamelMessageInfo *info, guint32 flags, guint32 set)
 				mi->flags |= CAMEL_GW_MESSAGE_JUNK | CAMEL_MESSAGE_JUNK | CAMEL_MESSAGE_JUNK_LEARN;
 		}
 
-
 		if (mi->summary && mi->summary->folder && mi->uid) {
 				CamelFolderChangeInfo *changes = camel_folder_change_info_new();
 
 				camel_folder_change_info_change_uid(changes, camel_message_info_uid(info));
-				camel_object_trigger_event(mi->summary->folder, "folder_changed", changes);
+				camel_folder_changed (mi->summary->folder, changes);
 				camel_folder_change_info_free(changes);
 				camel_folder_summary_touch(mi->summary);
 		}
@@ -431,7 +360,6 @@ gw_info_set_flags (CamelMessageInfo *info, guint32 flags, guint32 set)
 		return TRUE;
 }
 
-
 void
 camel_gw_summary_add_offline (CamelFolderSummary *summary, const gchar *uid, CamelMimeMessage *message, const CamelMessageInfo *info)
 {
@@ -440,7 +368,7 @@ camel_gw_summary_add_offline (CamelFolderSummary *summary, const gchar *uid, Cam
 	const CamelTag *tag;
 
 	/* Create summary entry */
-	mi = (CamelGroupwiseMessageInfo *)camel_folder_summary_info_new_from_message (summary, message);
+	mi = (CamelGroupwiseMessageInfo *)camel_folder_summary_info_new_from_message (summary, message, NULL);
 
 	/* Copy flags 'n' tags */
 	mi->info.flags = camel_message_info_flags(info);
@@ -496,11 +424,8 @@ groupwise_summary_clear (CamelFolderSummary *summary, gboolean uncache)
 	camel_folder_summary_clear_db (summary);
 	/*camel_folder_summary_save (summary);*/
 
-	if (uncache)
-		camel_data_cache_clear (((CamelGroupwiseFolder *) summary->folder)->cache, "cache", NULL);
-
 	if (camel_folder_change_info_changed (changes))
-		camel_object_trigger_event (summary->folder, "folder_changed", changes);
+		camel_folder_changed (summary->folder, changes);
 	camel_folder_change_info_free (changes);
 }
 
