@@ -69,6 +69,7 @@ struct _CamelIMAPXServer {
 	CamelIMAPXStream *stream;
 	struct _capability_info *cinfo;
 	gboolean is_ssl_stream;
+	gboolean is_process_stream;
 
 	CamelIMAPXNamespaceList *nsl;
 
@@ -91,16 +92,16 @@ struct _CamelIMAPXServer {
 
 	/* info on currently selected folder */
 	CamelFolder *select_folder;
-	gchar *select;
 	CamelFolderChangeInfo *changes;
 	CamelFolder *select_pending;
 	guint32 permanentflags;
-	guint32 uidvalidity;
 	guint32 unseen;
+	guint64 uidvalidity;
+	guint64 highestmodseq;
+	guint32 uidnext;
 	guint32 exists;
 	guint32 recent;
 	guint32 mode;
-	guint32 unread;
 
 	/* any expunges that happened from the last command, they are
 	   processed after the command completes. */
@@ -133,13 +134,13 @@ CamelIMAPXServer *
 		camel_imapx_server_new		(CamelStore *store,
 						 CamelURL *url);
 gboolean	camel_imapx_server_connect	(CamelIMAPXServer *is,
-						 gint state,
 						 CamelException *ex);
 gboolean	imapx_connect_to_server		(CamelIMAPXServer *is,
 						 CamelException *ex);
 GPtrArray *	camel_imapx_server_list		(CamelIMAPXServer *is,
 						 const gchar *top,
 						 guint32 flags,
+						 const gchar *ext,
 						 CamelException *ex);
 void		camel_imapx_server_refresh_info	(CamelIMAPXServer *is,
 						 CamelFolder *folder,
