@@ -28,26 +28,37 @@
 #include <glib/gi18n-lib.h>
 #include "e-destination-store.h"
 
-#define ITER_IS_VALID(destination_store, iter) ((iter)->stamp == (destination_store)->stamp)
-#define ITER_GET(iter)                         GPOINTER_TO_INT (iter->user_data)
-#define ITER_SET(destination_store, iter, index)      \
-G_STMT_START {                                        \
-	(iter)->stamp = (destination_store)->stamp;   \
-	(iter)->user_data = GINT_TO_POINTER (index);  \
-} G_STMT_END
+#define ITER_IS_VALID(destination_store, iter) \
+	((iter)->stamp == (destination_store)->priv->stamp)
+#define ITER_GET(iter) \
+	GPOINTER_TO_INT (iter->user_data)
+#define ITER_SET(destination_store, iter, index) \
+	G_STMT_START { \
+	(iter)->stamp = (destination_store)->priv->stamp; \
+	(iter)->user_data = GINT_TO_POINTER (index); \
+	} G_STMT_END
+
+#define E_DESTINATION_STORE_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), E_TYPE_DESTINATION_STORE, EDestinationStorePrivate))
+
+struct _EDestinationStorePrivate {
+	GPtrArray *destinations;
+	gint stamp;
+};
 
 static GType column_types [E_DESTINATION_STORE_NUM_COLUMNS];
 
 static void e_destination_store_tree_model_init (GtkTreeModelIface *iface);
 
-G_DEFINE_TYPE_EXTENDED (EDestinationStore, e_destination_store, G_TYPE_OBJECT, 0,
+G_DEFINE_TYPE_EXTENDED (
+	EDestinationStore, e_destination_store, G_TYPE_OBJECT, 0,
 	G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL, e_destination_store_tree_model_init);
 	column_types [E_DESTINATION_STORE_COLUMN_NAME]    = G_TYPE_STRING;
 	column_types [E_DESTINATION_STORE_COLUMN_EMAIL]   = G_TYPE_STRING;
 	column_types [E_DESTINATION_STORE_COLUMN_ADDRESS] = G_TYPE_STRING;
-	)
+)
 
-static void         e_destination_store_finalize        (GObject            *object);
 static GtkTreeModelFlags e_destination_store_get_flags       (GtkTreeModel       *tree_model);
 static gint         e_destination_store_get_n_columns   (GtkTreeModel       *tree_model);
 static GType        e_destination_store_get_column_type (GtkTreeModel       *tree_model,
@@ -79,21 +90,50 @@ static gboolean     e_destination_store_iter_parent     (GtkTreeModel       *tre
 static void destination_changed (EDestinationStore *destination_store, EDestination *destination);
 static void stop_destination    (EDestinationStore *destination_store, EDestination *destination);
 
-/* ------------------ *
- * Class/object setup *
- * ------------------ */
+static void
+destination_store_dispose (GObject *object)
+{
+	EDestinationStorePrivate *priv;
+	gint ii;
 
-static GObjectClass *parent_class = NULL;
+	priv = E_DESTINATION_STORE_GET_PRIVATE (object);
+
+	for (ii = 0; ii < priv->destinations->len; ii++) {
+		EDestination *destination;
+
+		destination = g_ptr_array_index (priv->destinations, ii);
+		stop_destination (E_DESTINATION_STORE (object), destination);
+		g_object_unref (destination);
+	}
+	g_ptr_array_set_size (priv->destinations, 0);
+
+	/* Chain up to parent's dispose() method. */
+	G_OBJECT_CLASS (e_destination_store_parent_class)->dispose (object);
+}
+
+static void
+destination_store_finalize (GObject *object)
+{
+	EDestinationStorePrivate *priv;
+
+	priv = E_DESTINATION_STORE_GET_PRIVATE (object);
+
+	g_ptr_array_free (priv->destinations, TRUE);
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (e_destination_store_parent_class)->finalize (object);
+}
 
 static void
 e_destination_store_class_init (EDestinationStoreClass *class)
 {
 	GObjectClass *object_class;
 
-	parent_class = g_type_class_peek_parent (class);
-	object_class = (GObjectClass *) class;
+	g_type_class_add_private (class, sizeof (EDestinationStorePrivate));
 
-	object_class->finalize = e_destination_store_finalize;
+	object_class = G_OBJECT_CLASS (class);
+	object_class->dispose = destination_store_dispose;
+	object_class->finalize = destination_store_finalize;
 }
 
 static void
@@ -116,27 +156,10 @@ e_destination_store_tree_model_init (GtkTreeModelIface *iface)
 static void
 e_destination_store_init (EDestinationStore *destination_store)
 {
-	destination_store->stamp        = g_random_int ();
-	destination_store->destinations = g_ptr_array_new ();
-}
-
-static void
-e_destination_store_finalize (GObject *object)
-{
-	EDestinationStore *destination_store = E_DESTINATION_STORE (object);
-	gint               i;
-
-	for (i = 0; i < destination_store->destinations->len; i++) {
-		EDestination *destination = g_ptr_array_index (destination_store->destinations, i);
-
-		stop_destination (destination_store, destination);
-		g_object_unref (destination);
-	}
-
-	g_ptr_array_free (destination_store->destinations, TRUE);
-
-	if (G_OBJECT_CLASS (parent_class)->finalize)
-		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	destination_store->priv =
+		E_DESTINATION_STORE_GET_PRIVATE (destination_store);
+	destination_store->priv->destinations = g_ptr_array_new ();
+	destination_store->priv->stamp = g_random_int ();
 }
 
 /**
@@ -144,16 +167,12 @@ e_destination_store_finalize (GObject *object)
  *
  * Creates a new #EDestinationStore.
  *
- * Return value: A new #EDestinationStore.
+ * Returns: A new #EDestinationStore.
  **/
 EDestinationStore *
 e_destination_store_new (void)
 {
-	EDestinationStore *destination_store;
-
-	destination_store = E_DESTINATION_STORE (g_object_new (E_TYPE_DESTINATION_STORE, NULL));
-
-	return destination_store;
+	return g_object_new (E_TYPE_DESTINATION_STORE, NULL);
 }
 
 /* ------------------ *
@@ -208,10 +227,15 @@ row_changed (EDestinationStore *destination_store, gint n)
 static gint
 find_destination_by_pointer (EDestinationStore *destination_store, EDestination *destination)
 {
+	GPtrArray *array;
 	gint i;
 
-	for (i = 0; i < destination_store->destinations->len; i++) {
-		EDestination *destination_here = g_ptr_array_index (destination_store->destinations, i);
+	array = destination_store->priv->destinations;
+
+	for (i = 0; i < array->len; i++) {
+		EDestination *destination_here;
+
+		destination_here = g_ptr_array_index (array, i);
 
 		if (destination_here == destination)
 			return i;
@@ -223,12 +247,18 @@ find_destination_by_pointer (EDestinationStore *destination_store, EDestination
 static gint
 find_destination_by_email (EDestinationStore *destination_store, EDestination *destination)
 {
+	GPtrArray *array;
 	gint i;
 	const gchar *e_mail = e_destination_get_email (destination);
 
-	for (i = 0; i < destination_store->destinations->len; i++) {
-		EDestination *destination_here = g_ptr_array_index (destination_store->destinations, i);
-		const gchar *mail = e_destination_get_email (destination_here);
+	array = destination_store->priv->destinations;
+
+	for (i = 0; i < array->len; i++) {
+		EDestination *destination_here;
+		const gchar *mail;
+
+		destination_here = g_ptr_array_index (array, i);
+		mail = e_destination_get_email (destination_here);
 
 		if (g_str_equal (e_mail, mail))
 			return i;
@@ -280,19 +310,21 @@ destination_changed (EDestinationStore *destination_store, EDestination *destina
  *
  * Gets the #EDestination from @destination_store at @iter.
  *
- * Return value: An #EDestination.
+ * Returns: An #EDestination.
  **/
 EDestination *
 e_destination_store_get_destination (EDestinationStore *destination_store, GtkTreeIter *iter)
 {
+	GPtrArray *array;
 	gint index;
 
 	g_return_val_if_fail (E_IS_DESTINATION_STORE (destination_store), NULL);
 	g_return_val_if_fail (ITER_IS_VALID (destination_store, iter), NULL);
 
+	array = destination_store->priv->destinations;
 	index = ITER_GET (iter);
 
-	return g_ptr_array_index (destination_store->destinations, index);
+	return g_ptr_array_index (array, index);
 }
 
 /**
@@ -301,19 +333,24 @@ e_destination_store_get_destination (EDestinationStore *destination_store, GtkTr
  *
  * Gets a list of all the #EDestinations in @destination_store.
  *
- * Return value: A #GList of pointers to #EDestination. The list is owned
+ * Returns: A #GList of pointers to #EDestination. The list is owned
  * by the caller, but the #EDestination elements aren't.
  **/
 GList *
 e_destination_store_list_destinations (EDestinationStore *destination_store)
 {
 	GList *destination_list = NULL;
+	GPtrArray *array;
 	gint   i;
 
 	g_return_val_if_fail (E_IS_DESTINATION_STORE (destination_store), NULL);
 
-	for (i = 0; i < destination_store->destinations->len; i++) {
-		EDestination *destination = g_ptr_array_index (destination_store->destinations, i);
+	array = destination_store->priv->destinations;
+
+	for (i = 0; i < array->len; i++) {
+		EDestination *destination;
+
+		destination = g_ptr_array_index (array, i);
 		destination_list = g_list_prepend (destination_list, destination);
 	}
 
@@ -335,6 +372,8 @@ void
 e_destination_store_insert_destination (EDestinationStore *destination_store,
 					gint index, EDestination *destination)
 {
+	GPtrArray *array;
+
 	g_return_if_fail (E_IS_DESTINATION_STORE (destination_store));
 	g_return_if_fail (index >= 0);
 
@@ -345,18 +384,18 @@ e_destination_store_insert_destination (EDestinationStore *destination_store,
 
 	g_object_ref (destination);
 
-	index = MIN (index, destination_store->destinations->len);
+	array = destination_store->priv->destinations;
+	index = MIN (index, array->len);
 
-	g_ptr_array_set_size (destination_store->destinations,
-			      destination_store->destinations->len + 1);
+	g_ptr_array_set_size (array, array->len + 1);
 
-	if (destination_store->destinations->len - 1 - index > 0) {
-		memmove (destination_store->destinations->pdata + index + 1,
-			 destination_store->destinations->pdata + index,
-			 (destination_store->destinations->len - 1 - index) * sizeof (gpointer));
+	if (array->len - 1 - index > 0) {
+		memmove (array->pdata + index + 1,
+			 array->pdata + index,
+			 (array->len - 1 - index) * sizeof (gpointer));
 	}
 
-	destination_store->destinations->pdata [index] = destination;
+	array->pdata[index] = destination;
 	start_destination (destination_store, destination);
 	row_inserted (destination_store, index);
 }
@@ -372,6 +411,8 @@ e_destination_store_insert_destination (EDestinationStore *destination_store,
 void
 e_destination_store_append_destination (EDestinationStore *destination_store, EDestination *destination)
 {
+	GPtrArray *array;
+
 	g_return_if_fail (E_IS_DESTINATION_STORE (destination_store));
 
 	if (find_destination_by_email (destination_store, destination) >= 0 && !e_destination_is_evolution_list (destination)) {
@@ -379,11 +420,12 @@ e_destination_store_append_destination (EDestinationStore *destination_store, ED
 		return;
 	}
 
+	array = destination_store->priv->destinations;
 	g_object_ref (destination);
 
-	g_ptr_array_add (destination_store->destinations, destination);
+	g_ptr_array_add (array, destination);
 	start_destination (destination_store, destination);
-	row_inserted (destination_store, destination_store->destinations->len - 1);
+	row_inserted (destination_store, array->len - 1);
 }
 
 /**
@@ -397,6 +439,7 @@ e_destination_store_append_destination (EDestinationStore *destination_store, ED
 void
 e_destination_store_remove_destination (EDestinationStore *destination_store, EDestination *destination)
 {
+	GPtrArray *array;
 	gint n;
 
 	g_return_if_fail (E_IS_DESTINATION_STORE (destination_store));
@@ -410,7 +453,8 @@ e_destination_store_remove_destination (EDestinationStore *destination_store, ED
 	stop_destination (destination_store, destination);
 	g_object_unref (destination);
 
-	g_ptr_array_remove_index (destination_store->destinations, n);
+	array = destination_store->priv->destinations;
+	g_ptr_array_remove_index (array, n);
 	row_deleted (destination_store, n);
 }
 
@@ -418,21 +462,23 @@ void
 e_destination_store_remove_destination_nth (EDestinationStore *destination_store, gint n)
 {
 	EDestination *destination;
+	GPtrArray *array;
 
-	g_return_if_fail ( n >= 0);
+	g_return_if_fail (n >= 0);
 
-	destination = g_ptr_array_index(destination_store->destinations, n);
+	array = destination_store->priv->destinations;
+	destination = g_ptr_array_index (array, n);
 	stop_destination (destination_store, destination);
 	g_object_unref (destination);
 
-	g_ptr_array_remove_index (destination_store->destinations, n);
+	g_ptr_array_remove_index (array, n);
 	row_deleted (destination_store, n);
 }
 
 guint
 e_destination_store_get_destination_count (EDestinationStore *destination_store)
 {
-	return destination_store->destinations->len;
+	return destination_store->priv->destinations->len;
 }
 
 /* ---------------- *
@@ -468,14 +514,19 @@ e_destination_store_get_column_type (GtkTreeModel *tree_model,
 static gboolean
 e_destination_store_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreePath *path)
 {
-	EDestinationStore *destination_store = E_DESTINATION_STORE (tree_model);
-	gint               index;
+	EDestinationStore *destination_store;
+	GPtrArray *array;
+	gint index;
 
 	g_return_val_if_fail (E_IS_DESTINATION_STORE (tree_model), FALSE);
 	g_return_val_if_fail (gtk_tree_path_get_depth (path) > 0, FALSE);
 
+	destination_store = E_DESTINATION_STORE (tree_model);
+
 	index = gtk_tree_path_get_indices (path)[0];
-	if (index >= destination_store->destinations->len)
+	array = destination_store->priv->destinations;
+
+	if (index >= array->len)
 		return FALSE;
 
 	ITER_SET (destination_store, iter, index);
@@ -512,7 +563,7 @@ e_destination_store_iter_next (GtkTreeModel  *tree_model,
 
 	index = ITER_GET (iter);
 
-	if (index + 1 < destination_store->destinations->len) {
+	if (index + 1 < destination_store->priv->destinations->len) {
 		ITER_SET (destination_store, iter, index + 1);
 		return TRUE;
 	}
@@ -534,7 +585,7 @@ e_destination_store_iter_children (GtkTreeModel *tree_model,
 		return FALSE;
 
 	/* But if parent == NULL we return the list itself as children of the root. */
-	if (destination_store->destinations->len <= 0)
+	if (destination_store->priv->destinations->len <= 0)
 		return FALSE;
 
 	ITER_SET (destination_store, iter, 0);
@@ -562,7 +613,7 @@ e_destination_store_iter_n_children (GtkTreeModel *tree_model,
 	g_return_val_if_fail (E_IS_DESTINATION_STORE (tree_model), -1);
 
 	if (iter == NULL)
-		return destination_store->destinations->len;
+		return destination_store->priv->destinations->len;
 
 	g_return_val_if_fail (ITER_IS_VALID (destination_store, iter), -1);
 	return 0;
@@ -581,7 +632,7 @@ e_destination_store_iter_nth_child (GtkTreeModel *tree_model,
 	if (parent)
 		return FALSE;
 
-	if (n < destination_store->destinations->len) {
+	if (n < destination_store->priv->destinations->len) {
 		ITER_SET (destination_store, iter, n);
 		return TRUE;
 	}
@@ -604,11 +655,12 @@ e_destination_store_get_value (GtkTreeModel *tree_model,
 			       GValue       *value)
 {
 	EDestinationStore *destination_store = E_DESTINATION_STORE (tree_model);
-	EDestination      *destination;
-	const gchar       *string;
-	GString	  *string_new;
-	gint               row;
-	EContact	  *contact;
+	EDestination *destination;
+	GString *string_new;
+	EContact *contact;
+	GPtrArray *array;
+	const gchar *string;
+	gint row;
 
 	g_return_if_fail (E_IS_DESTINATION_STORE (tree_model));
 	g_return_if_fail (column < E_DESTINATION_STORE_NUM_COLUMNS);
@@ -616,11 +668,13 @@ e_destination_store_get_value (GtkTreeModel *tree_model,
 
 	g_value_init (value, column_types [column]);
 
+	array = destination_store->priv->destinations;
+
 	row = ITER_GET (iter);
-	if (row >= destination_store->destinations->len)
+	if (row >= array->len)
 		return;
 
-	destination = g_ptr_array_index (destination_store->destinations, row);
+	destination = g_ptr_array_index (array, row);
 	g_assert (destination);
 
 	switch (column) {
@@ -661,3 +715,11 @@ e_destination_store_get_value (GtkTreeModel *tree_model,
 			break;
 	}
 }
+
+gint
+e_destination_store_get_stamp (EDestinationStore *destination_store)
+{
+	g_return_val_if_fail (E_IS_DESTINATION_STORE (destination_store), 0);
+
+	return destination_store->priv->stamp;
+}
